@@ -30,9 +30,12 @@ from elyra.metadata.handlers import SchemaspaceResourceHandler
 from elyra.metadata.manager import MetadataManager
 from elyra.metadata.schema import SchemaManager
 from elyra.metadata.storage import FileMetadataCache
+from elyra.pipeline.catalog_connector import ComponentCatalogConnector
+from elyra.pipeline.component_catalog import ComponentCatalog
 from elyra.pipeline.handlers import PipelineComponentHandler
 from elyra.pipeline.handlers import PipelineComponentPropertiesHandler
 from elyra.pipeline.handlers import PipelineExportHandler
+from elyra.pipeline.handlers import PipelineRuntimeTypesHandler
 from elyra.pipeline.handlers import PipelineSchedulerHandler
 from elyra.pipeline.handlers import PipelineValidationHandler
 from elyra.pipeline.processor import PipelineProcessor
@@ -54,12 +57,12 @@ class ElyraApp(ExtensionAppJinjaMixin, ExtensionApp):
     extension_url = '/lab'
     load_other_extensions = True
 
-    classes = [FileMetadataCache, MetadataManager, PipelineProcessor]
+    classes = [FileMetadataCache, MetadataManager, PipelineProcessor, ComponentCatalogConnector, ComponentCatalog]
 
     # Local path to static files directory.
-    # static_paths = [
-    #     DEFAULT_STATIC_FILES_PATH
-    # ]
+    static_paths = [
+        os.path.join(DEFAULT_STATIC_FILES_PATH, "icons"),
+    ]
 
     # Local path to templates directory.
     # template_paths = [
@@ -71,25 +74,34 @@ class ElyraApp(ExtensionAppJinjaMixin, ExtensionApp):
     def initialize_handlers(self):
         schemaspace_regex = r"(?P<schemaspace>[\w\.\-]+)"
         resource_regex = r"(?P<resource>[\w\.\-]+)"
-        path_regex = r"(?P<path>[\w\.\/\-\%]+)"
+        path_regex = r"(?P<path>(?:(?:/[^/]+)+|/?))"  # same as jupyter server and will include a leading slash
         processor_regex = r"(?P<processor>[\w]+)"
-        component_regex = r"(?P<component_id>[\w\.\-]+)"
+        component_regex = r"(?P<component_id>[\w\.\-:]+)"
 
         self.handlers.extend([
+            # API
             (f'/{self.name}/{YamlSpecHandler.get_resource_metadata()[0]}', YamlSpecHandler),
+
+            # Content
+            (f'/{self.name}/contents/properties{path_regex}', ContentHandler),
+
+            # Metadata
             (f'/{self.name}/metadata/{schemaspace_regex}', MetadataHandler),
             (f'/{self.name}/metadata/{schemaspace_regex}/{resource_regex}', MetadataResourceHandler),
             (f'/{self.name}/schema/{schemaspace_regex}', SchemaHandler),
             (f'/{self.name}/schema/{schemaspace_regex}/{resource_regex}', SchemaResourceHandler),
             (f'/{self.name}/schemaspace', SchemaspaceHandler),
             (f'/{self.name}/schemaspace/{schemaspace_regex}', SchemaspaceResourceHandler),
-            (f'/{self.name}/pipeline/schedule', PipelineSchedulerHandler),
-            (f'/{self.name}/pipeline/export', PipelineExportHandler),
+
+            # Pipeline
             (f'/{self.name}/pipeline/components/{processor_regex}', PipelineComponentHandler),
             (f'/{self.name}/pipeline/components/{processor_regex}/{component_regex}/properties',
              PipelineComponentPropertiesHandler),
-            (f'/{self.name}/contents/properties/{path_regex}', ContentHandler),
-            (f'/{self.name}/elyra/pipeline/validate', PipelineValidationHandler),
+            (f'/{self.name}/pipeline/export', PipelineExportHandler),
+            (f'/{self.name}/pipeline/runtimes/types', PipelineRuntimeTypesHandler),
+            (f'/{self.name}/pipeline/schedule', PipelineSchedulerHandler),
+            (f'/{self.name}/pipeline/validate', PipelineValidationHandler),
+
         ])
 
     def initialize_settings(self):
